import 'dart:convert';
import 'dart:io';

import 'package:flutter_vizzy_lib/Md5Util.dart';
import 'package:flutter_vizzy_lib/VizzyApiClient.dart';
import 'package:flutter_vizzy_lib/VizzyService.dart';
import 'package:flutter_driver/flutter_driver.dart';
import 'package:meta/meta.dart';

class Vizzy {

  String userEmail;
  String userToken;
  String vizzyUrl; // Important to finish with /
  int projectId;

  String commitSha;
  String pullRequestNumber;

  var imagesDir = Directory("vizzy");

  FlutterDriver driver;

  VizzyService apiService;

  Vizzy({@required FlutterDriver driver, @required int projectId, String url, String username, String token}) {
    this.driver = driver;
    initializeParameters(url, username, token);
    this.projectId = projectId;
    apiService = VizzyService(
        baseUrl: vizzyUrl,
        userEmail: userEmail,
        userToken: userToken);
    prepareScreenshotDir();
  }

  void initializeParameters(String url, String username, String token) {
    // Environment variables seems to be the easiest way
    // Defaulting for now to our configuration
    // In the future, finish with an error if not configured
    userEmail = username ?? Platform.environment['VIZZY_EMAIL'];
    userToken = token ?? Platform.environment['VIZZY_TOKEN'];
    vizzyUrl = url ?? Platform.environment['VIZZY_URL'];

    commitSha = Platform.environment['COMMIT_SHA'];
    pullRequestNumber = Platform.environment['PULL_REQUEST_NUMBER'];
    // If there is a PR number, it is not a dev build
  }

  void prepareScreenshotDir() {
    if (imagesDir.existsSync()) {
      imagesDir.deleteSync(recursive: true);
    }
    imagesDir.createSync(recursive: true);
  }

  Future takeScreenshot([String screenshotName]) async {
    final filePath = "${imagesDir.path}/$screenshotName.png";
    final file = await File(filePath).create(recursive: true);
    final pixels = await driver.screenshot();
    await file.writeAsBytes(pixels);
  }

  Future verifyImages() async {
    BuildInfo buildInfo = await apiService.createBuild(projectId, commitSha, pullRequestNumber);

    var imagesToUpload = await filterImagesToUpload(buildInfo.id);

    await uploadRequestedImages(buildInfo.id, imagesToUpload);

    await apiService.commitBuild(buildInfo.id);
  }

  Future<List<String>> filterImagesToUpload(int buildId) async {
    var imageMd5Request = await calculateMd5s();
    var imageMd5response = await apiService.postImageMd5s(buildId, imageMd5Request);
    return imageMd5response.imageMd5s.keys.toList();
  }

  Future<ImageMd5Request> calculateMd5s() async {
    var allFiles = await getAllFilesUnderData(imagesDir);
    var imagesMap = Map<String, String>();
    for(File file in allFiles) {
      String md5 = await calculateMD5File(file);
      // We get the path from the app directory, remove that part
      var imagePath = file.path.replaceFirst("${imagesDir.path}/", "");
      imagesMap[imagePath] = md5;
    }
    var imageMd5Request = ImageMd5Request();
    imageMd5Request.imageMd5s = json.encode(imagesMap);

    return imageMd5Request;
  }

  Future<List<File>> getAllFilesUnderData(Directory imagesDir) async {
    var allImages = List<File>();
    await for (FileSystemEntity entity in imagesDir.list(recursive: true, followLinks: false)) {
      allImages.add(File(entity.path));
    }
    return allImages;
  }

  Future uploadRequestedImages(int buildId, List<String> imagesToUpload) async {
    for(String fileName in imagesToUpload) {
      File file = File("${imagesDir.path}/$fileName");
      await apiService.uploadImage(buildId, file, fileName);
    }
  }
}
